
# q-expansion of newform 1002.2.a.j, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1002
weight = 2
poly_data = [-11, 31, 20, -16, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[-7, 0, 1, 0, 0], 2], [[-3, -11, 1, 1, 0], 2], [[17, 8, -14, 0, 1], 4]]

hecke_ring_character_values = None
aps_data = [[-1, 0, 0, 0, 0], [1, 0, 0, 0, 0], [0, 0, 1, 0, 0], [2, 0, 0, 0, -1], [0, 0, 0, 1, 0], [1, 1, 0, 0, 0], [1, -1, 0, 0, 0], [2, 0, 0, -1, 0], [0, 0, 0, -1, 0], [-2, 2, 0, -1, 0], [2, 0, 0, 1, 1], [1, 1, -1, 1, 0], [1, -1, -2, 1, 0], [4, -2, 0, 1, 2], [-2, 2, 0, 0, 1], [0, 0, -3, -1, -2], [3, -3, -1, -1, -2], [4, -2, 0, 0, -2], [2, 0, -1, 2, 2], [-2, 2, 2, -1, -2], [2, 0, 2, 1, 2], [1, 1, 2, -1, 0], [1, -1, -1, 0, -4], [-2, 2, 0, 0, 3], [-2, 4, 2, 0, 1], [0, 0, -1, 3, 0], [-1, 3, 0, -1, 0], [0, 0, 2, 0, 4], [3, -1, 0, -2, 0], [1, -1, -2, -1, -2], [4, -2, 2, 1, 3], [-1, 1, -2, -2, 2], [0, 0, -2, -2, 1], [0, 2, 1, 0, 4], [-2, 2, -1, 3, 0], [1, 1, 0, 1, -2], [6, -4, -2, -2, -2], [6, -4, 1, 1, 0], [-1, 0, 0, 0, 0], [-2, -4, -2, -2, 0], [2, -2, -4, 1, 0], [-6, -4, 2, -1, 2], [-6, 0, 0, 3, 1], [-10, 0, -2, 1, 4], [-2, 2, 2, -1, 0], [-2, -2, -2, -4, 2], [0, 2, 0, -2, -4], [-10, 0, -2, 0, -5], [-1, 1, 3, 1, 4], [-10, 0, 0, 2, 2], [0, 0, 2, 1, 1], [-6, 0, -2, 0, -4], [-8, -2, 4, -1, 2], [4, -4, -2, -1, 0], [3, -3, -2, 2, 0], [-4, -2, 0, 0, 7], [-6, 6, 3, 2, -2], [-9, -1, -4, 0, 4], [1, 1, 3, -2, -4], [-8, -4, 2, -1, 5], [0, 2, -2, 2, 0], [-8, -4, 0, -2, 4], [0, 2, -3, 1, -4], [0, 0, 2, 1, -1], [-10, 0, -2, 1, 2], [-8, 2, -4, -3, 2], [0, 2, -4, -3, 2], [12, -4, 0, 1, -3], [13, -1, -1, -3, 2], [-19, -3, 1, 1, 0], [4, -4, 4, 7, -1], [2, 4, 2, -4, 0], [-8, 4, 2, 2, 6], [-5, 7, -1, -2, -2], [0, 2, 3, -3, -2], [8, 4, -2, 0, 2], [-8, -4, -8, -1, -2], [8, 6, 2, 3, -4], [5, -5, 0, 3, 0], [2, 0, -4, 0, 0], [0, 0, -4, 4, 4], [-4, -6, -6, -1, 4], [12, 0, 8, 1, -3], [-16, -6, -2, 1, 1], [1, 1, -4, -3, -6], [11, 1, -3, -5, -2], [-4, 4, 0, 8, -1], [-2, 4, 0, -2, 0], [-4, -2, -2, -3, -4], [3, -1, 2, -3, -2], [20, 4, 4, -2, 2], [-10, -2, 2, 2, 2], [5, -3, 2, -2, 0], [6, 6, 4, 0, 4], [4, -2, 4, -5, 2], [8, 4, -6, 4, 2], [-10, 4, 2, 0, 4], [-3, 3, -2, 0, 4], [18, -4, -2, -3, 0], [1, 1, 5, -4, 0], [0, 2, 9, 6, 2], [2, -2, 2, -5, 2], [-4, 4, 0, -5, -2], [-13, 1, 4, -2, 4], [6, 8, 3, 3, -2], [6, 2, 4, 2, -3], [-9, -3, 3, -2, 10], [23, 1, 2, 0, 4], [-12, -6, 6, 0, 5], [-18, -4, -4, 0, -1], [27, -1, 0, 3, 4], [0, 2, 10, 7, 6], [8, -2, -2, 2, -6], [6, -4, 2, -1, -8], [-4, 6, 10, -3, -3], [-3, 3, -2, -1, 8], [18, -4, 4, 1, -6], [4, -4, -10, -2, -2], [-14, -4, 8, 4, 2], [-3, 3, -5, 2, 4], [-15, 5, 0, -2, 8], [6, -4, 4, 1, -2], [26, -2, -2, -1, -2], [-17, 5, 4, -2, 0], [-2, 4, 0, 5, 6], [2, 4, 4, -2, 2], [-7, -3, -3, 2, -2], [-12, 0, 0, -8, -4], [1, 1, 0, 4, -2], [16, -2, 2, -3, -4], [4, -2, -3, 1, 2], [0, 6, -8, -2, -3], [9, -7, -2, -2, 0], [8, -6, -6, -3, 0], [14, 10, -4, -1, -1], [-18, -4, -12, -3, 4], [8, 4, -5, 1, -6], [16, -2, 0, -5, -2], [-4, 4, 6, 1, 0], [8, 4, -2, 2, -7], [0, 2, -11, 0, 2], [2, -2, 6, 5, 0], [3, 11, -2, 4, -2], [-9, -3, -2, 2, 10], [-9, -1, 7, -6, -2], [-26, -4, 0, -1, 1], [-18, -4, -12, 2, 2], [6, 6, 0, 9, -1], [10, 4, 0, 5, -6], [6, -12, -4, 2, -2], [22, -8, 0, -1, 6], [9, 3, 0, 1, -8], [14, 0, 2, 1, -8], [18, -6, 6, 6, 8], [2, 0, 12, -3, -2], [0, 0, 4, 5, -5], [2, 0, 2, 2, 11], [-14, 2, 6, -7, -7], [20, -6, -2, 1, -6], [-4, 4, -2, -7, -4], [2, -2, 8, -10, -6], [9, 3, 0, -2, -6], [6, 2, -4, 2, 4], [33, 3, 1, 2, 2], [9, 3, 14, 0, 6], [-2, -10, -6, 2, 6], [5, -3, 0, 0, 6], [2, 0, -4, -1, -4], [6, 2, 10, 0, -3], [22, -4, -4, -2, -4], [-21, -3, -3, 6, -6], [0, 2, 10, -8, -6], [-4, -2, 10, -2, -5], [8, -6, 0, 1, 9], [11, -9, -6, 5, 2], [-20, 2, 6, -2, -2], [20, 0, 2, -2, 6], [20, -8, 3, 4, 12], [-2, 10, 8, 6, 0], [9, 5, 5, -2, -8], [10, -8, 4, 5, -5], [16, -4, -2, 0, -4], [21, -7, -1, -2, -2], [25, -1, 6, 2, 2], [-26, 2, 2, -1, 4], [-10, -2, -4, 3, 4], [-8, -2, -6, -7, -8], [14, -6, -6, -7, -4], [-20, 10, -2, -5, 5], [2, 10, 4, 3, 4], [40, -2, 2, 2, 8], [3, -3, 13, 10, 2], [12, 8, 6, 6, 4], [-8, 2, -8, 7, 8], [-24, 0, -8, 1, -12], [-8, -4, 6, -10, -5], [-2, 10, 2, 0, -3], [-12, 14, 4, 6, 4], [-2, 2, -2, 10, 1], [2, -8, -6, -1, 1], [-16, -8, 6, -5, -4], [-22, -12, 6, -5, -3], [23, 3, 3, -6, 0], [20, -6, -14, -4, -8], [-3, 3, 0, 0, -8], [2, -14, -10, 3, 2], [1, 1, -2, -10, 4], [-4, 4, 0, -3, -6], [7, -7, 0, 4, -8], [-4, -12, -2, 4, 2], [-16, 6, 8, 7, 11], [-8, 2, 14, 3, 2], [-13, 15, 4, 0, 4], [-7, 7, 2, 2, 2], [-30, 0, -4, -4, -2], [-28, -6, 8, -3, -1], [13, -11, 0, -6, -12], [12, 6, -2, 2, -2], [12, 0, 4, -9, -1], [-4, 4, -5, 4, 2], [13, 1, 2, 2, -2], [24, 2, -4, 3, 7], [17, 7, 4, 3, -8], [-10, -6, -16, 0, 0], [5, -5, 2, 8, 2], [20, 6, 0, 2, 0], [16, 8, 16, 0, 1], [-2, -10, -4, -2, 0], [-1, -9, -16, 0, -4], [4, -4, 6, -2, -4], [9, 5, 1, 0, -6], [8, 6, -2, 1, 0], [-3, 5, -12, -6, -12], [-3, -9, 2, -10, -16], [12, 8, 12, -8, -2], [22, 2, -6, -3, -6], [-2, 4, 2, 16, 0], [22, 2, 4, 5, -10], [1, -1, -1, 6, -2], [-2, -16, -4, -2, 6], [-19, -5, -6, 4, 6], [16, -8, -2, 0, 2], [13, 13, 8, 1, 6], [7, -5, 3, 0, -2], [4, 2, 8, 10, 12], [20, -8, -6, -2, -18], [-30, -4, -6, -2, -5], [-11, -1, -5, -3, -4], [20, 0, 0, 4, 2], [8, 16, -4, -2, -2], [20, -6, -2, 8, 10], [-4, 10, 10, 4, 6], [-24, 0, -2, -4, -8], [14, 0, 14, 7, 2], [-6, 6, 2, -11, -8], [-20, -4, 4, 3, 2], [-7, 9, 5, -1, 14], [-14, 10, -2, -1, 10], [12, 12, 3, -5, 10], [-4, 0, 10, 1, 7], [-29, 7, -6, -6, 0], [29, 7, 1, -2, -4], [-9, 11, -1, 1, 10], [1, 1, 7, 3, -4], [8, -2, 4, 6, -4], [18, -10, -12, -1, 2], [-20, 8, -10, -9, 4], [-3, -9, -4, -5, 0], [30, -4, 7, 10, -6], [20, 4, 10, -4, 0], [25, 1, 2, -8, 2], [4, -2, -2, -2, 6], [-10, -12, -12, 5, 4], [4, 10, -6, 10, 13], [26, 6, -8, 0, -6], [-19, -3, -6, -4, -12], [-25, 1, 5, -6, 2], [11, 3, -7, 11, 6], [-28, -6, -4, 1, 6], [32, 4, 10, 2, 6], [-16, 4, 2, -6, -6], [-15, 5, -2, 9, 0], [4, -4, -12, -5, -10], [8, 6, 2, 2, 2], [-4, 6, 11, 5, 2], [-26, 2, 4, 3, -4], [24, 2, 8, -1, -15], [28, -16, -10, -1, 4], [12, 14, -2, -5, 3], [21, -9, 0, 4, 0], [-10, -8, -4, -10, 0], [-17, 17, 3, -5, 2], [-18, 6, 8, -4, 7], [-24, 12, 4, 3, -2], [46, -8, -2, 12, 14], [-20, 8, 2, 2, 2], [14, 0, 4, -4, -19], [4, 8, 3, 1, -6], [5, -5, 15, 8, 4], [-8, 10, -10, -8, -4], [24, 2, -14, -2, -14], [0, 0, 21, 10, 10], [12, -10, 4, -4, -9], [-15, -9, 6, -14, -6], [20, 0, 4, 4, -16], [-10, 0, 12, 10, 4], [23, -23, -11, -4, -4], [-2, -8, 8, -5, -12], [-14, 14, 4, 16, -2], [12, 2, -8, -1, 4], [38, 10, 6, 4, -12], [-16, -2, -6, 3, -18], [-2, 8, 14, 6, 10], [-24, 2, -2, -7, -8], [-16, 4, 2, -3, -6], [-10, 6, 8, 4, 10], [13, 11, -3, 7, 6], [-8, -10, -6, 0, 1], [-4, 6, 12, -6, 4], [11, -11, -8, 3, 0], [-32, -2, 0, -6, -14], [-10, 12, -4, 4, 1], [8, 16, -4, -6, 6], [-5, -5, -8, -5, -10], [7, -7, -6, -1, -2], [-18, 8, 16, 2, 9], [-4, -6, 8, 0, 20], [14, -6, -12, -7, -6], [18, 6, 16, -6, 3], [-2, 2, 4, 5, -10], [-24, -10, -8, 1, -6], [-30, -6, -10, 3, 0], [13, 1, 0, 2, 10], [-18, -6, 0, 3, -4], [10, -20, 0, 3, 10], [-8, 8, 14, 7, 8], [-20, -2, -2, -12, -16], [23, 1, 0, -9, 8], [-34, 0, 2, 11, 2], [-18, -4, -6, -1, -19], [-32, 10, -2, -5, -6], [6, 0, -14, 0, -2], [20, -8, -5, 3, -8], [-15, -7, 4, 10, -6], [-16, -2, 6, -3, 4], [-18, -6, 2, -6, 4], [30, 8, 6, 2, -10], [-20, -2, -16, -6, 2], [0, 0, -6, -7, 8], [-12, 12, 12, 6, 14], [6, 2, 8, -11, 2], [-10, -12, -8, -12, 6], [2, 10, -5, 2, -6], [-1, -9, 2, 9, -10], [13, 1, -3, 6, -8], [-3, 3, 4, -1, 18], [4, -16, -6, -3, 5], [55, 5, -3, 7, 4], [-5, 5, -8, -6, 2], [10, -4, 6, 11, -5], [-28, -6, -8, 15, -10], [9, 15, -10, -1, 2], [0, 12, 2, 6, -8], [8, 4, -2, 8, 0], [-8, 22, 3, 5, 0], [-38, 4, -4, 0, 12], [22, 2, 11, -4, 2], [19, 7, 0, 5, 6], [0, 8, 2, -3, 5], [43, 5, 5, 4, -2], [14, 12, 4, 15, -2], [-38, 8, 0, -10, -18], [-22, 4, -8, -2, 2], [15, -13, -10, -7, -18], [19, 7, 5, 4, 12], [21, -9, -2, 8, 10], [-2, 2, 14, -1, -8], [0, -10, 4, 1, 17], [19, 5, 16, 1, 2], [2, -18, -4, -8, 2], [-12, 0, 24, 4, 18], [-4, 4, -8, -5, -11], [1, 1, -2, -1, 4], [-28, 16, -6, -11, -7], [26, -6, 4, 7, 4], [26, 10, -6, 9, -8], [1, 1, -6, -17, 4], [11, 3, 17, -3, 12], [-36, 8, 2, 4, 14], [42, 6, 10, 3, -14], [-34, 12, -4, -3, 12], [0, 12, -12, -8, 6], [-12, 24, 14, -1, 0], [-32, -2, -7, -3, 4], [22, 2, 4, -8, 6], [-32, -14, 2, -3, 12], [8, -6, -6, 2, 7], [-36, 0, -8, -10, 3], [58, -8, 3, 5, 12], [32, 4, 19, 2, 8], [14, -12, -14, -4, 4], [0, -6, 14, -4, -10], [19, -17, -10, 8, 2], [-5, -7, -14, -8, -28], [-20, -4, 9, -1, 16], [43, -5, 6, -6, 4], [-19, 9, -1, 7, 20], [9, 15, 0, -3, 16], [-26, -8, -7, 3, -12], [8, 16, -6, 9, 20], [-38, -8, 6, -12, -4], [36, -12, 0, 11, 18], [-44, 8, 16, 5, 8], [26, 6, -14, 1, -8], [-38, -8, -6, 0, -18], [52, -4, 2, -8, -2], [2, 22, 10, -4, 12], [-8, 10, -10, -13, -1], [58, 2, 6, -3, -7], [2, -2, 6, 12, -3], [-4, -8, 10, 1, 16], [-11, 13, -3, -3, 12], [70, 2, 0, 2, -3], [52, -4, 2, -3, -2], [-16, -6, -2, -15, -3], [42, -6, 2, -1, -12], [14, -2, -2, 10, 18], [-38, -10, 12, -8, 3], [0, 14, 2, 0, 4], [18, 6, 6, -7, -2]]
